#ifndef LINE_H_
#define LINE_H_

#include <QString>
#include <QGraphicsItem>
#include "Shape.h"
#include "Point.h"


/**
 * \brief Classe Linha, extende Shape, e possui dois pontos.
 * Uma Linha tem também um comprimento
 */
class Line: public Shape {
public:
	Line();

	/**
	 * Construtor que não recebe o nome do objeto,
	 * e é usado somente para fins de teste.
	 * \param a Ponto inicial da linha
	 * \param b Ponto final da linha
	 */
	Line(Point,Point);

	/**
	 * Construtor usado quando uma linha é criada através
	 * da GUI. O nome da linha é passado.
	 * \param id O nome da linha
	 * \param a Ponto inicial da linha
	 * \param b Ponto final da linha
	 */
	Line(QString,Point,Point);

        //metodo que retorna o ponto a da linha
	Point getA();
        Point* getAP();
        // metodo que retorna o ponto b da linha
	Point getB();
        Point* getBP();
        //metodos que seta o ponto as pontos da reta
        void setA(Point*);
        void setB(Point*);
        //metodo que realiza o calculo da coordenadas de pontos de projeção
        void calcCpp(double);
        void saveXML(ofstream*);
        // Calcula o comprimento da linha, usando a fórmula clássica de pitágoras
	double length();
        // Igualdade de linhas. Duas linhas são iguais se seus respectivos pontos o são
	bool operator==(const Line&) const;
        // Negação lógica da igualdade
	bool operator!=(const Line&) const;

	virtual ~Line();
private:
	Point a,b;
};

#endif /* LINE_H_ */
