
#include "Mediator.h"



void Mediator::loadFile(QString fn) {

    //std::string file(fn);
    TiXmlDocument doc(fn.toAscii());
    bool loadOkay = doc.LoadFile();
    if (loadOkay)
    {
            XMLLoader::processShapes(doc);
    }
    else
    {
            qDebug() << "Failed to load file!";
    }
    doc.Clear();
}

void Mediator::saveFile(QString fn) {

    ofstream myfile;
    myfile.open(fn.toAscii(), ios::out | ios::trunc);
    qDebug()<< fn.toAscii();
    if (myfile.is_open())
    {
            myfile <<  "<?xml version=";
            myfile<< "\"1.0\"";
            myfile<<"?>\n<formas>";

            QList<Shape*> shapeList =  displayFile->getShapes();
            for (int i = 0; i< shapeList.length(); i++)
            {
                shapeList.at(i)->saveXML(&myfile);
            }
            myfile<<"\n</formas>";
            myfile.close();
    }
    else
    {
            qDebug() << "Failed to create the file!";
    }
}

//metodo que retorna a unica instacia mantendo o objeto como um singleton
Mediator* Mediator::instance() {
	static Mediator uniqueInstance;
	return &uniqueInstance;
}

//metodo construtor
Mediator::Mediator() {
	displayFile = new DisplayFile();
	janelaPrincipal = new JanelaPrincipal;
	window = new Window(*(janelaPrincipal->getCanvas()));
	QObject::connect(displayFile, SIGNAL(shapeAdded()),
					 window, SLOT(redraw()));

	janelaPrincipal->setListModel(displayFile->getStringListModel());
	janelaPrincipal->show();
        selectedItem = NULL;
}

//metodo destrutor
Mediator::~Mediator() {
	delete displayFile;
	delete window;
	delete janelaPrincipal;
}

//metodo que retorna a lista dos objetos presentes no displayFile
QList<Shape*> Mediator::getShapes() {
	return displayFile->getShapes();
}

//metodo que chama o metodo para a criação do ponto
void Mediator::addPoint(double x, double y, QString name)
{
	Point *addedPoint = ShapeFactory::instance()->createPoint(x,y,name);
        displayFile->addShape(addedPoint);
}

//metodo que chama o metodo para a criação do ponto
void Mediator::addPoint(double x, double y, QString name, int R,int G,int B)
{
        Point *addedPoint = ShapeFactory::instance()->createPoint(x,y,name);
        addedPoint->setRGB(R,G,B);
        displayFile->addShape(addedPoint);
}

//metodo que chama o metodo para a criação da linha
void Mediator::addLine(double x1, double y1, double x2, double y2, QString name) {
	Line *addedLine = ShapeFactory::instance()->createLine(x1,y1,x2,y2,name);
        displayFile->addShape(addedLine);
}

void Mediator::addLine(double x1, double y1, double x2, double y2, QString name, int R, int G, int B) {
        Line *addedLine = ShapeFactory::instance()->createLine(x1,y1,x2,y2,name);
        addedLine->setRGB(R,G,B);
        displayFile->addShape(addedLine);
}

//metodo que chama o metodo para a criação do poligono
void Mediator::addPolygon(QList<double> xs, QList<double> ys, QString name, bool preenchido) {
        Polygon *addedPolygon = ShapeFactory::instance()->createPolygon(xs,ys,name, preenchido);
        displayFile->addShape(addedPolygon);
}

void Mediator::addPolygon(QList<double> xs, QList<double> ys, QString name, bool preenchido, int R,int G, int B) {
        Polygon *addedPolygon = ShapeFactory::instance()->createPolygon(xs,ys,name, preenchido);
        addedPolygon->setRGB(R,G,B);
        displayFile->addShape(addedPolygon);
}

//metodo que chama o metodo para a criação da bezier
void Mediator::addBezier(QList<double>xs, QList<double>ys, QString name)
{
    Bezier *addedBezier = ShapeFactory::instance()->createBezier(xs,ys,name);
    displayFile->addShape(addedBezier);
}

void Mediator::addBezier(QList<double>xs, QList<double>ys, QString name,int R,int G, int B)
{
    Bezier *addedBezier = ShapeFactory::instance()->createBezier(xs,ys,name);
    addedBezier->setRGB(R,G,B);
    displayFile->addShape(addedBezier);
}

//metodo que chama o metodo para a criação da spline
void Mediator::addBspline(QList<double> xs, QList<double>ys, QString name)
{
   Bspline *addedSpline = ShapeFactory::instance()->createBspline(xs,ys,name);
   displayFile->addShape(addedSpline);
}

void Mediator::addBspline(QList<double> xs, QList<double>ys, QString name,int R,int G, int B)
{
   Bspline *addedSpline = ShapeFactory::instance()->createBspline(xs,ys,name);
   addedSpline->setRGB(R,G,B);
   displayFile->addShape(addedSpline);
}

//metodo que chama o metodo para realizar o zoomIn
void Mediator::zoomIn(double valor) {
        window->zoomIn(valor);
}

//metodo que chama o metodo para realizar o zoomOut
void Mediator::zoomOut(double valor) {
        window->zoomOut(valor);
}

//metodo que chama o metodo para realizar a movimentação vertical(direita esquerda conforme o valor)
void Mediator::verticalMove(int valor) {
        window->moveWindow(0,valor);
}

//metodo que chama o metodo para realizar a movimentação horizontal(cima baixo conforme o valor)
void Mediator::horizontalMove(int valor) {
        window->moveWindow(valor,0);
}

//metodo que atribui o item selecionado do display file a uma variavel
void Mediator::setSelectedItem(Shape *item)
{
    selectedItem = item;
}

//metodo que chama o metodo que realiza o redesenho dos objetos
void Mediator::redraw()
{
    window->redraw();
}

//metodo que retorna o item selecionado
Shape* Mediator::getSelectedItem()
{
    return selectedItem;
}

//metodo que realiza a rotação da window
void Mediator::rotate(double angle) {
    window->rotate(angle);
}
//metodo que retorna a instancia da window
Window* Mediator::getWindow() {
    return window;
}


