#ifndef POINT_H_
#define POINT_H_

#include <QString>
#include "Shape.h"

/**
 * \brief Classe ponto, extende uma Shape e usa coordenadas inteiras
 */
class Point: public Shape {
public:
	Point();

	/**
	 * Construtor que não recebe nome do objeto, usado somente para testes.
	 * \param x Abcissa
	 * \param y Ordenada
	 */
        Point(double,double);

	/**
	 * Construtor que recebe o nome do objeto, usado quando
	 * o ponto é criado através da GUI.
	 * \param id Nome do objeto
	 * \param x Abcissa
	 * \param y Ordenada
	 */
        Point(QString,double,double);
        // Acessor da abcissa do ponto
        double getX() const;
        // Define o ponto X
        void setX(double);
        // Acessor da ordenada do ponto
        double getY() const;
        // Define o ponto y
        void setY(double);
        //metodo que retorna a coordenada x dos pontos de projeção
        double getXcpp();
        //metodo que retorna a coordenada y dos pontos de projeção
        double getYcpp();
        //metodo que atribui a coordenada x dos pontos de projeção
        void setXcpp(double);
        //metodo que atribui a coordenada y dos pontos de projeção
        void setYcpp(double);
        //metodo que calculo a coordenada de ponto de projeção do objeto
        void calcCpp(double);
        void saveXML(ofstream*);
        void xmlPonto(ofstream*);
        // Igualdade entre pontos. Dois pontos são iguais se suas coordenadas o são
	bool operator==(const Point&) const;
        // Negação lógica da igualdade
	bool operator!=(const Point&) const;
        // Soma de pontos. Soma das respectivas coordenadas
	Point operator+(const Point&) const;
        // Diferença entre pontos. Funciona de forma análoga à soma.
	Point operator-(const Point&) const;

	virtual ~Point();
private:
        //variaveis que aramzenam o x e o y do objeto e suas coordenadas de pontos de projeção
        double x, y, ycpp,xcpp;

};

#endif /* POINT_H_ */
