#include "Polygon.h"
#include "transformer.h"

//metodo construtor
Polygon::Polygon() : Shape(POLYGON) {
}

Polygon::Polygon(const QList<Point> &points) : Shape(POLYGON) {
	this->points = points;

}

Polygon::Polygon(QString id, const QList<Point> &points) : Shape(POLYGON) {
	this->points = points;
	this->name = "pol_" + id;
}

//metodo destrutor
Polygon::~Polygon() {
}

//metodo que retorna a lista de pontos que forma o poligono
QList<Point> Polygon::getPoints() {
	return points;
}

void Polygon::setPoints(QList<Point> points){
    this->points = points;
}

//metodo que compara se um poligono é igual a outro poligono
bool Polygon::operator==(const Polygon &other) const {
	return points == other.points;
}

//metodo que compara se um poligono é diferente a outro poligono
bool Polygon::operator!=(const Polygon &other) const {
	return !((*this) == other);
}
//metodo que adiciona pontos ao poligono
void Polygon::addPoint(const Point &p) {
	points << p;
}

bool Polygon::isFilled()
{
    return fill;
}

void Polygon::setFill(bool value)
{
    this->fill = value;
}

//metodo que calcula as coordenadas de ponto de projeção do objeto
void Polygon::calcCpp(double angle)
{
    Point origem;
    origem.setX(0);
    origem.setY(0);
    QList<Point> n = Transformer::rotation(getPoints(), origem, angle);
    QList<Point> l;
    for(int i = 0; i < n.size(); i++) {
        Point p = points.at(i);
        Point pn = n.at(i);
        p.setXcpp(pn.getX());
        p.setYcpp(pn.getY());
        l.append(p);
    }
    setPoints(l);
}

void Polygon::saveXML(ofstream* f)
{
    std::stringstream ss;
    char saida[7];
    char p[2];
    p[0] = 0;
    p[1] = 0;
    p[2] = 0;
    if(this->R != 0){
        ss << std::hex << this->R;
        ss >> p;
        saida[0] = p[0];
        saida[1] = p[1];
        ss.clear();
    }else{
     saida[0] = '0';
     saida[1] = '0';
    }
    if(this->G != 0){
        ss << std::hex << this->G;
        ss >> p;
        saida[2] = p[0];
        saida[3] = p[1];
        ss.clear();
    }else
    {
        saida[2] = '0';
        saida[3] = '0';
    }
    if(this->B != 0){
        ss << std::hex<< this->B;
        ss >> p;
        saida[4] = p[0];
        saida[5] = p[1];
    }else
    {
        saida[4] = '0';
        saida[5] = '0';
    }
    saida[6] = 0;
    (*f)<<"\n\t<poligono nome=\"";
    (*f)<<this->name.toStdString();
    (*f)<<"\" cor=\"";
    (*f)<<saida;
    if(this->isFilled())
        (*f)<<"\"cheio=\"verdadeiro\">\n";
    else
        (*f)<<"\"cheio=\"falso\">\n";
    QList <Point> pontos = this->getPoints();
    for(int i =0; i<pontos.length(); i++)
    {
         Point p = pontos.at(i);
         p.xmlPonto(f);
    }
    (*f)<<"\t</poligono>";
}
