#ifndef POLYGON_H_
#define POLYGON_H_

#include <QList>
#include <QGraphicsItem>
#include <QString>
#include "Shape.h"
#include "Point.h"
#include "math.h"

/**
 * \brief Classe polígono. É uma lista de pontos, e como também
 * extende Shape, Transformerpossui nome e é do tipo POLYGON.
 */
class Polygon: public Shape {
public:
	Polygon();

	/**
	 * Construtor para fins de teste, que não recebe o nome do polígono.
	 * \param points Lista de pontos pertencentes ao polígono, passada por referência
	 */
	Polygon(const QList<Point>&);

	/**
	 * Construtor utilizado quando um polígono é criado pelo usuário, através
	 * da GUI, passando o nome do polígono.
	 * \param id O nome do polígono
	 * \param points Lista de pontos pertencentes ao polígono
	 */
	Polygon(QString, const QList<Point>&);
        virtual ~Polygon();
        //metodo que retorna a lista de pontos que formam o poligono
	QList<Point> getPoints();
        //metodo que adiciona pontos a lista
	void addPoint(const Point&);
        //metodo que atribui a lista de pontos do poligono
        void setPoints(QList<Point>);
        //metodo que ve se o poligno é preenchido ou não
        bool isFilled();
        void setFill(bool);
        //metodo que calcula as coordenadas de ponto de projeção do objeto
        void calcCpp(double);
        void saveXML(ofstream*);
        // Igualdade de polígonos. Polígonos são iguais se seus pontos o são
        bool operator==(const Polygon&) const;
        // Negação lógica da igualdade
        bool operator!=(const Polygon&) const;

private:
        //variavel que armazena os pontos que formam o poligono
        bool fill;
	QList<Point> points;
};

#endif /* POLYGON_H_ */
