#ifndef SHAPEFACTORY_H_
#define SHAPEFACTORY_H_

#include <QList>
#include "Point.h"
#include "Line.h"
#include "Polygon.h"
#include "Bezier.h"
#include "bspline.h"

/**
 * \brief Classe responsável pela criação e configuração das formas
 * geométricas utilizadas no sistema.Padrão de projeto conhecido como Factory.
 */
class ShapeFactory {
public:
        // Obtém a instância (única) da classe
	static ShapeFactory* instance();

        // Cria um ponto e retorna uma referẽncia do ponto
        Point* createPoint(double,double,QString);

        // Cria uma linha e retorna uma referência da linha
        Line* createLine(double,double,double,double,QString);

        // Cria um poligono e retorna uma referência do poligono
        Polygon* createPolygon(QList<double>, QList<double>, QString,bool);
        //metodo que cria a curva de bezier
        Bezier* createBezier(QList<double> xs, QList<double> ys, QString id);
        //metodo que cria a curva spline
        Bspline* createBspline(QList<double> xs, QList<double> ys, QString id);

	virtual ~ShapeFactory();
private:
	ShapeFactory();
	ShapeFactory(const ShapeFactory&);
	ShapeFactory& operator=(const ShapeFactory&);
};

#endif /* SHAPEFACTORY_H_ */
