#include "model/Mediator.h"
#include "Window.h"
#include "Shape.h"
#include "Point.h"
#include "Line.h"
#include "Polygon.h"
#include "Bezier.h"
#include "math.h"
#include <QDebug>

//metodo construtor recebendo a area de desenho
Window::Window(QGraphicsView& cv) {
        canvas = &cv;

	x_min = y_min = 0;
	x_max = y_max = viewportSize;

        viewPort = new ViewPort(viewportSize, canvas);
}

Window::~Window() {
}
//metodo responsavel pela atualização da viewport desenhando os objetos conforme perspectiva
//utilizada (zoom e navegação)
void Window::redraw() {
        viewPort->redrawing(x_min, y_min, x_max, y_max);
	Point *p = 0;
	Line *l = 0;
        Polygon *pol = 0;
        Bezier *b = 0;
        Bspline *sp = 0;
        QPen pincel;

	QList<Shape*> toBeRedrawn = Mediator::instance()->getShapes();
	foreach(Shape* s, toBeRedrawn)
		{
			shape_t sType = s->getType();
                        pincel.setColor(QColor(s->R,s->G,s->B,255));
			if (sType == POINT) {
                                p = (Point*) s;
                                viewPort->addPoint(p, pincel);
			} else if (sType == LINE) {
                                l = (Line*) s;
                                viewPort->addLine(l,pincel);
                        } else if (sType == POLYGON) {
				pol = (Polygon*) s;
                                if(pol->isFilled())
                                    viewPort->addPolygon(pol,pincel,true);
                                else
                                    viewPort->addPolygon(pol,pincel,false);
                        } else if (sType == BEZIER) {
                            b = (Bezier*) s;
                            viewPort->addBezier(b,pincel);
                        } else if (sType == BSPLINE) {
                            sp = (Bspline*) s;
                            viewPort->addBspline(sp, pincel);
                        }
		}
        viewPort->updateSceneCanvas();
}

//metodo que realiza a transformação viewport
Point Window::transform(Point p) {

        double x = p.getXcpp(), y = p.getYcpp();
        double w_width = x_max - x_min, w_heigth = y_max - y_min;

        double xr = double((x - x_min) * viewportSize) / w_width;
        double yr = (1 - (double((y - y_min))/(w_heigth))) * viewportSize;

	return Point(xr,yr);
}
//metodo que realiza o Zoom In
void Window::zoomIn(double valor) {
    if(x_max-x_min>(valor*2) && y_max-y_min>(valor*2)){
        x_max -= valor;
        x_min +=valor;
        y_max -= valor;
        y_min +=valor;
        redraw();
    }
}
//metodo que realiza o Zoom out
void Window::zoomOut(double valor) {
        x_max += valor;
        x_min -=valor;
        y_max += valor;
        y_min -=valor;
	redraw();

}
//metodo que movimenta conforme o click nos botões
void Window::moveWindow(int variacaoX, int variacaoY) {
        x_min += variacaoX;
        x_max += variacaoX;
        y_min += variacaoY;
        y_max += variacaoY;
	redraw();
}

//metodo que realiza a rotação da window
void Window::rotate(double ang) {
    angle = angle + ang;
    QList<Shape*> toBeRedrawn = Mediator::instance()->getShapes();
    foreach(Shape* s, toBeRedrawn) {
        s->calcCpp(angle);
    }
    redraw();
}

//metodo que retorna o angulo atual da window
double Window::getAngle() {
     return angle;
}
