#ifndef WINDOW_H_
#define WINDOW_H_

#include <QObject>
#include <QGraphicsItem>
#include <QGraphicsView>
#include <QGraphicsScene>
#include "Point.h"
#include "viewport.h"

/**
 * \brief Classe que representa a porção do mundo atualmente "vista"
 * pelo usuário. As funcionalidades deslocamento e de zoom da cena
 * se dão através da mudança da localização e das dimensões da Window,
 * respectivamente.
 */
class Window : public QObject {
	Q_OBJECT
public:
	/**
	 * O construtor da Window precisa receber uma referência para a Viewport
	 * à qual está ligada (onde projeta os objetos).
	 * \param viewport Referência para o canvas no qual essa window projeta
	 * os objetos
	 */
	Window(QGraphicsView&);
        //metodo que realiza o zoom In
        void zoomIn(double);
        //metodo que realiza o zoom out
        void zoomOut(double);
        //metodo que move a window
	void moveWindow(int,int);
        //metodo que rotaciona a window
        void rotate(double);
        //metodo que retorna o angulo atual da window
        double getAngle();
        //vetor que informa a direção para cima em relação a visão do usuario
        double vup[3];

	virtual ~Window();

private:
	const static int viewportSize = 500;
        //metodo que realiza a transformação viewport
        Point transform(Point);
        //pontos maximos e minimos da viewport
        double x_max, y_max, x_min, y_min;
        //variavel que armazena a area sobre a qual é realizada os desenhos
	QGraphicsView *canvas;
        //variavel que armazena o angulo atual da window
        double angle;

        ViewPort *viewPort;

public slots:
        void redraw();
};

#endif /* WINDOW_H_ */
