#include "transformer.h"

//Metodo construtor
Transformer::Transformer()
{
}

//Metodo destrutor
Transformer::~Transformer()
{
}

//Metodo que realiza a rotação sobre a origem
void Transformer::rotationOrigin(Shape* s, int angle) {
    if(angle != 0)
    {
        double rad = angle * M_PI/180;
        double sen = sin(rad);
        double coss = cos(rad);
        double matrix[3][3] = {{coss,-sen,0},{sen,coss,0},{0,0,1}};
        double resposta[3];
        shape_t type = s->getType();
        if(type == POINT) {
            Point *p = 0;
            p = (Point*) s;
            double vector[3] = {p->getX(), p->getY(), 1};
            multiVetMatriz33(matrix, vector, resposta);
            p->setX(resposta[0]);
            p->setY(resposta[1]);

        } else if(type == LINE) {
            Line *l = 0;
            l = (Line*) s;
            Point* a = l->getAP();
            Point* b = l->getBP();
            Transformer::rotationOrigin(a, angle);
            Transformer::rotationOrigin(b, angle);

        } else if(type == POLYGON) {
            Polygon *p = 0;
            p = (Polygon*) s;
            QList<Point> pontosT;
            QList<Point> pontos = p->getPoints();
             foreach(Point point, pontos) {
                 Transformer::rotationOrigin(&point,angle);
                 pontosT.append(point);
            }
             p->setPoints(pontosT);
         } else if(type == BEZIER) {
            Bezier *p = 0;
            p = (Bezier*) s;
            QList<Point> pontosT;
            QList<Point> pontos = p->getRPoints();
            foreach(Point point, pontos) {
                 Transformer::rotationOrigin(&point,angle);
                 pontosT.append(point);
            }
            p->setRPoints(pontosT);
        }else if(type == BSPLINE)
        {
            Bspline *p = 0;
            p = (Bspline*) s;
            QList<Point> pontosT;
            QList<Point> pontos = p->getRPoints();
            foreach(Point point, pontos) {
                 Transformer::rotationOrigin(&point,angle);
                 pontosT.append(point);
             }
            p->setRPoints(pontosT);
        }
    }
}

//Metodo que realiza a rotação sobre o centro do objeto
void Transformer::rotationCenter(Shape* s, int angle) {
      shape_t type = s->getType();
      if(angle !=0)
      {
         if(type == POINT) {
            //rotação sobre o centro do ponto é o proprio ponto
        } else if(type == LINE) {
            Line *l = 0;
            l = (Line*) s;
            QList<Point> pl;
            pl.append(l->getA());
            pl.append(l->getB());
            Point media = Media(pl);
            pl = rotation(pl,media,angle);
            l->setA(&pl.takeAt(0));
            l->setB(&pl.takeAt(0));

        } else if(type == POLYGON) {
            Polygon *p = 0;
            p = (Polygon*) s;
            QList<Point> pontosT;
            QList<Point> pontos = p->getPoints();
            Point media = Media(pontos);
            pontosT = rotation(pontos,media,angle);
            p->setPoints(pontosT);
        } else if(type == BEZIER) {
            Bezier *p = 0;
            p = (Bezier*) s;
            QList<Point> pontosT;
            QList<Point> pontos = p->getRPoints();
            Point media = Media(pontos);
            pontosT = rotation(pontos,media,angle);
            p->setRPoints(pontosT);
        }else if(type == BSPLINE)
        {
            Bspline *p = 0;
            p = (Bspline*) s;
            QList<Point> pontosT;
            QList<Point> pontos = p->getRPoints();
            Point media = Media(pontos);
            pontosT = rotation(pontos,media,angle);
            p->setRPoints(pontosT);
        }
    }
}


//Metodo que realiza a rotação sobre um ponto escolhido
void Transformer::rotationPointasRef(Shape* s, int angle, int px,int py) {
    shape_t type = s->getType();
    if(angle != 0)
    {
        Point vetor;
        vetor.setX(px);
        vetor.setY(py);
        if(type == POINT) {
            Point *p = 0;
            p = (Point*) s;
            QList<Point> pontos;
            Point ress;
            pontos.append((*p));
            pontos = rotation(pontos,vetor,angle);
            ress = pontos.takeAt(0);
            p->setX(ress.getX());
            p->setY(ress.getY());
        } else if(type == LINE) {
            Line *l = 0;
            l = (Line*) s;
            QList<Point> pl;
            pl.append(l->getA());
            pl.append(l->getB());
            pl = rotation(pl,vetor,angle);
            l->setA(&pl.takeAt(0));
            l->setB(&pl.takeAt(0));
        } else if(type == POLYGON) {
            Polygon *p = 0;
            p = (Polygon*) s;
            QList<Point> pontosT;
            QList<Point> pontos = p->getPoints();
            pontosT = rotation(pontos,vetor,angle);
            p->setPoints(pontosT);
        } else if(type == BEZIER) {
            Bezier *p = 0;
            p = (Bezier*) s;
            QList<Point> pontosT;
            QList<Point> pontos = p->getRPoints();
            pontosT = rotation(pontos,vetor,angle);
            p->setRPoints(pontosT);
        }else if(type == BSPLINE)
        {
            Bspline *p = 0;
            p = (Bspline*) s;
            QList<Point> pontosT;
            QList<Point> pontos = p->getRPoints();
            pontosT = rotation(pontos,vetor,angle);
            p->setRPoints(pontosT);
        }
    }
}

//Metodo que realiza o escalonamento do objeto
void Transformer::scale(Shape* s, double scaleX, double scaleY) {
    shape_t type = s->getType();
    if(type == POINT) {
        Point *p = 0;
        p = (Point*) s;
        Point ress;
        QList<Point> pontos;
        pontos.append((*p));
        pontos = Scale(pontos,(*p),scaleX,scaleY);
        ress = pontos.takeAt(0);
        p->setX(ress.getX());
        p->setY(ress.getY());
    } else if(type == LINE) {
        Line *l = 0;
        l = (Line*) s;
        QList<Point> pl;
        QList<Point> ress;
        pl.append(l->getA());
        pl.append(l->getB());
        Point media = Media(pl);
        ress = Scale(pl,media,scaleX,scaleY);
        l->setA(&ress.takeAt(0));
        l->setB(&ress.takeAt(0));
    } else if(type == POLYGON) {
        Polygon *p = 0;
        p = (Polygon*) s;
        QList<Point> pontosT;
        QList<Point> pontos = p->getPoints();
        Point media = Media(pontos);
        pontosT = Scale(pontos,media,scaleX,scaleY);
        p->setPoints(pontosT);
    } else if(type == BEZIER) {
        Bezier *p = 0;
        p = (Bezier*) s;
        QList<Point> pontosT;
        QList<Point> pontos = p->getRPoints();
        Point media = Media(pontos);
        pontosT = Scale(pontos,media,scaleX,scaleY);
        p->setRPoints(pontosT);
    }else if(type == BSPLINE) {
        Bezier *p = 0;
        p = (Bezier*) s;
        QList<Point> pontosT;
        QList<Point> pontos = p->getRPoints();
        Point media = Media(pontos);
        pontosT = Scale(pontos,media,scaleX,scaleY);
        p->setRPoints(pontosT);
        }
}

//Metodo que realiza a translação do objeto
void Transformer::translation(Shape* s, int dx, int dy) {
    shape_t type = s->getType();
    double matrix[3][3] = {{1,0,0},{0,1,0},{dx,dy,1}};
    double resposta[3] = {0,0,1};
    if(type == POINT) {
        Point *p = 0;
        p = (Point*) s;
        double vector[3] = {p->getX(), p->getY(), 1};
        multiVetMatriz33(matrix, vector, resposta);
        p->setX(resposta[0]);
        p->setY(resposta[1]);
    } else if(type == LINE) {
        Line *l = 0;
        l = (Line*) s;
        Point* a = l->getAP();
        Point* b = l->getBP();
        Transformer::translation(a, dx, dy);
        Transformer::translation(b, dx, dy);

    } else if(type == POLYGON) {
        Polygon *p = 0;
        p = (Polygon*) s;
        QList<Point> pontosT;
        QList<Point> pontos = p->getPoints();
         foreach(Point point, pontos) {
             Transformer::translation(&point, dx, dy);
             pontosT.append(point);
        }
         p->setPoints(pontosT);
    } else if(type == BEZIER) {
        Bezier *p = 0;
        p = (Bezier*) s;
        QList<Point> pontosT;
        QList<Point> pontos = p->getRPoints();
         foreach(Point point, pontos) {
             Transformer::translation(&point, dx, dy);
             pontosT.append(point);
        }
         p->setRPoints(pontosT);
    }
    else if(type == BSPLINE){
        Bezier *p = 0;
        p = (Bezier*) s;
        QList<Point> pontosT;
        QList<Point> pontos = p->getRPoints();
         foreach(Point point, pontos) {
             Transformer::translation(&point, dx, dy);
             pontosT.append(point);
             }
         p->setRPoints(pontosT);
        }
}

//Metodo que realiza a multiplicação de um vetor por uma matriz 3x3
void Transformer::multiVetMatriz33(double mat[3][3], double vec[3], double res[3]) {
    for (int column = 0; column < 3; column++) {
        res[column] = 0;
        for (int element = 0; element < 3; element++) {
            res[column] += vec[element] * mat[element][column];
        }
    }
}

//Metodo que realiza os calculos da rotação
QList<Point> Transformer::rotation(QList<Point> pontos, Point vetor, int angle)
{
    double rad = angle * M_PI/180;
    double sen = sin(rad);
    double coss = cos(rad);
    double matrixRot[3][3] = {{coss,-sen,0},{sen,coss,0},{0,0,1}};
    double matrix[3][3] = {{0,0,0},{0,0,0},{0,0,0}} ;
    double matrixTra[3][3] = {{1,0,0},{0,1,0},{-vetor.getX(),-vetor.getY(),1}};
    double resposta[3];
    QList<Point> rpontos;
    MatMultMat33(matrixTra, matrixRot,matrix);
    matrixTra[2][0] = -matrixTra[2][0];
    matrixTra[2][1] = -matrixTra[2][1];
    MatMultMat33(matrix, matrixTra, matrixRot);
    foreach(Point p, pontos)
    {
        double vector[3] = {p.getX(), p.getY(), 1};
        multiVetMatriz33(matrixRot, vector, resposta);
        p.setX(resposta[0]);
        p.setY(resposta[1]);
        rpontos.append(p);
    }
        return rpontos;
}

//Metodo que realiza os calculos de escalonamento
QList<Point> Transformer::Scale(QList<Point> pontos,Point media, double scaleX, double scaleY)
{
    double matrixSca[3][3] = {{scaleX,0,0},{0,scaleY,0},{0,0,1}};
    double matrix[3][3] = {{0,0,0},{0,0,0},{0,0,0}} ;
    double resposta[3];
    double matrixTra[3][3] = {{1,0,0},{0,1,0},{-media.getX(),-media.getY(),1}};
    QList<Point> rpontos;
    MatMultMat33(matrixTra, matrixSca,matrix);
    matrixTra[2][0] = -matrixTra[2][0];
    matrixTra[2][1] = -matrixTra[2][1];
    MatMultMat33(matrix, matrixTra, matrixSca);

    foreach(Point p, pontos)
    {
        double vector[3] = {p.getX(), p.getY(), 1};
        multiVetMatriz33(matrixSca, vector, resposta);
        p.setX(resposta[0]);
        p.setY(resposta[1]);
        rpontos.append(p);
    }
    return rpontos;
}

//Metodo que realiza a multiplicação de matrizes 3x3
void Transformer::MatMultMat33(double Mat1[3][3], double Mat2[3][3], double Res[3][3])
{
    int n, m,p;
    for (n=0;n<3;n++)
        for (m=0;m<3;m++)
             Res[n][m] = 0;

   for (n=0;n<3;n++)
       for (m=0;m<3;m++)
           for (p=0;p<3;p++)
               Res[n][m] += Mat1[n][p] * Mat2[p][m];
}
void Transformer::MatMultMat42 (double Mat1[4][4], double Mat2[4][2], double Res[4][2])
{
    //TODO: VERIFICAR SE TA CORRETO
    int n, m,p;
    for (n=0;n<4;n++)
        for (m=0;m<2;m++)
             Res[n][m] = 0;

   for (n=0;n<4;n++)
       for (m=0;m<2;m++)
           for (p=0;p<4;p++)
               Res[n][m] += Mat1[n][p] * Mat2[p][m];
}
//Metodo que calcula a media dos pontos informados
Point Transformer::Media(QList<Point> pontos)
{
    Point media;
    double mediaX = 0;
    double mediaY = 0;
    int n = 0;
    foreach(Point point, pontos)
    {
        mediaX += point.getX();
        mediaY += point.getY();
        n++;
    }
    mediaX = mediaX/n;
    mediaY = mediaY/n;
    media.setX(mediaX);
    media.setY(mediaY);

    return media;
}
