#include "viewport.h"
#include <QDebug>

ViewPort::ViewPort(double viewportSize, QGraphicsView* cv)
{
    canvas = cv;
    mySize = viewportSize - 100;
    viewPortSize = viewportSize;
    scene = new QGraphicsScene(0, 0, viewportSize, viewportSize);
    rectSize = 50;
    scene->addRect(rectSize, rectSize, mySize, mySize);
    updateSceneCanvas();
 }


ViewPort::~ViewPort() {
}

void ViewPort::redrawing(double wxminm,double wyminm,double wxmaxm,double wymaxm)
{
    wxmin = wxminm; wymin = wyminm;
    wxmax = wxmaxm; wymax = wymaxm;
    scene = new QGraphicsScene(0, 0, viewPortSize, viewPortSize);
    scene->addRect(rectSize, rectSize, mySize, mySize);
}

void ViewPort::addBezier(Bezier *b, QPen pen)
{
    QList<Point> BPoints = b->generatePoints(0.01);
    for(int i = 1; i < BPoints.size(); i++)
    {
        Line *l = new Line(BPoints.value(i-1),BPoints.value(i));
        addLine(l,pen);
    }

}

void ViewPort::addBspline(Bspline *b, QPen pen)
{
    QList<Point> SPoints = b->generatePoints(0.01);
    for(int i = 1; i < SPoints.size(); i++)
    {
        Line *l = new Line(SPoints.value(i-1),SPoints.value(i));
        addLine(l,pen);
    }

}

void ViewPort::addPolygon(Polygon *pol, QPen pen,bool preenchido) {
    QList<Point> polP = pol->getPoints();
    if(preenchido){
        // FAZ O CLIPPING ANTES DE ADICIONAR
            QList <QPoint> pointsWindow;
            pointsWindow << QPoint (wxmax,wymax);
            pointsWindow << QPoint (wxmin,wymax);
            pointsWindow << QPoint (wxmax,wymin);
            pointsWindow << QPoint (wxmin,wymin);


            //passo final transforma o poligono em qpolf e adiciona a cena preenchido
            QPolygonF poly;
            double pontos [4];
            Point a,b;
            bool confirm =false;
            for(int i = 0; i< polP.size()-1; i++)
            {

                Line *l = new Line(polP.at(i),polP.at(i+1));
                confirm = ViewPort::addLinee(l,pontos);
                if(confirm){
                    a.setXcpp(pontos[0]);
                    a.setYcpp(pontos[1]);

                    b.setXcpp(pontos[2]);
                    b.setYcpp(pontos[3]);
                    Point pa = transform(a);
                    Point pb = transform(b);
                    poly << QPoint(pa.getX(), pa.getY());
                    poly << QPoint(pb.getX(), pb.getY());
                }
                //teste do triangulo

            }
            Line  *l = new Line(polP.at(polP.size()-1),polP.at(0));
            confirm = ViewPort::addLinee(l,pontos);

            if(confirm){
                a.setXcpp(pontos[0]);
                a.setYcpp(pontos[1]);

                b.setXcpp(pontos[2]);
                b.setYcpp(pontos[3]);
                Point pa = transform(a);
                Point pb = transform(b);
                poly << QPoint(pa.getX(), pa.getY());
                poly << QPoint(pb.getX(), pb.getY());
            }
            scene->addPolygon(poly, pen,pen.brush());
    }else{
        for(int i = 0; i< polP.size()-1; i++)
        {
            Line *l = new Line(polP.at(i),polP.at(i+1));
            ViewPort::addLine(l,pen);
        }
        Line  *l = new Line(polP.at(polP.size()-1),polP.at(0));
        ViewPort::addLine(l,pen);
    }
}

void ViewPort::updateSceneCanvas() {
    canvas->setScene(scene);
}


void ViewPort::addPoint(Point* p, QPen pen) {
    if(isInside(p)){
        Point Ptransf = transform(*p);
        double x=Ptransf.getX(),y=Ptransf.getY();
        scene->addLine(x,y,x,y,pen);
    }
}

bool ViewPort::isInside(Point* p)
{
    bool inside = false;
    if((wxmin<= p->getXcpp())&&(p->getXcpp()<= wxmax))
        if((wymin<= p->getYcpp())&&(p->getYcpp() <= wymax))
        inside = true;

    return inside;
}

bool ViewPort::clipTesteLine(double ps[4], double qs[4], double *u1, double *u2)
{
    double r;
    for(int k=0; k<4; k++) {
        r = qs[k]/ps[k];
        if(ps[k]==0 && qs[k]<0) return false;

        if(ps[k]<0) {
            if(r>(*u2)) return false;
            else if(r>(*u1)) (*u1)=r;
        } else if(ps[k]>0) {
            if(r< (*u1)) return false;
            else if(r<(*u2)) (*u2)=r;
        }
    }
    return true;
}

void ViewPort::addLine(Line* l, QPen pen) {
    double x1, y1, x2, y2;
    double ps[4];
    double qs[4];

    double u1 = 0;
    double u2 = 1;

    x1 = l->getA().getXcpp();
    y1 = l->getA().getYcpp();
    x2 = l->getB().getXcpp();
    y2 = l->getB().getYcpp();

    double dx = x2 - x1;
    double dy = y2 - y1;

    ps[0] = -dx;
    ps[1] = dx;
    ps[2] = -dy;
    ps[3] = dy;
    qs[0] =  x1 - wxmin;
    qs[1] = wxmax - x1;
    qs[2] = y1 - wymin;
    qs[3] = wymax - y1;

    if(clipTesteLine(ps,qs, &u1, &u2))
    {
        x2 = x1 + u2*dx;
        y2 = y1 + u2*dy;
        x1 = x1 + u1*dx;
        y1 = y1 + u1*dy;

        Point a,b;
        a.setXcpp(x1);
        a.setYcpp(y1);

        b.setXcpp(x2);
        b.setYcpp(y2);

        Point pa = transform(a);
        Point pb = transform(b);

        scene->addLine(pa.getX(), pa.getY(), pb.getX(), pb.getY(), pen);
    }
}

bool ViewPort::addLinee(Line* l, double pontos [4]) {

        double x1, y1, x2, y2;
        double ps[4];
        double qs[4];

        double u1 = 0;
        double u2 = 1;

        x1 = l->getA().getXcpp();
        y1 = l->getA().getYcpp();
        x2 = l->getB().getXcpp();
        y2 = l->getB().getYcpp();

       double dx = x2 - x1;
    double dy = y2 - y1;

    ps[0] = -dx;
    ps[1] = dx;
    ps[2] = -dy;
    ps[3] = dy;
    qs[0] =  x1 - wxmin;
    qs[1] = wxmax - x1;
    qs[2] = y1 - wymin;
    qs[3] = wymax - y1;

    if(clipTesteLine(ps,qs, &u1, &u2))
    {
        x2 = x1 + u2*dx;
        y2 = y1 + u2*dy;
        x1 = x1 + u1*dx;
        y1 = y1 + u1*dy;

        pontos[0] = x1;

        pontos[1] = y1;
        pontos[2] = x2;
        pontos[3] = y2;

        return true;

    }else
    return false;
}

double ViewPort::min(double a, double b) {
    if(a>=b)
        return (b);
    else
        return (a);
}

double ViewPort::max(double a, double b) {
    if(a<=b)
        return (b);
    else
        return (a);
}

//metodo que realiza a transformação viewport
Point ViewPort::transform(Point p) {

        double x = p.getXcpp(), y = p.getYcpp();
        double w_width = wxmax - wxmin, w_heigth = wymax - wymin;

        double xr = rectSize+ double((x - wxmin) * mySize) / w_width;
        double yr = rectSize+ (1 - (double((y - wymin))/(w_heigth))) * mySize;

        return Point(xr,yr);
}
